# Define _make_verbose if it doesn't exist (RHEL8)
%{!?_make_verbose:%define _make_verbose V=1 VERBOSE=1}

Name:           nginx-mod-otel
Version:        0.1.2
Release:        1%{?dist}
Summary:        Nginx OpenTelemetry module
License:        2-clause BSD-like license
URL:            https://github.com/nginxinc/nginx-otel
Packager:       Teddysun <i@teddysun.com>
Source100:      https://github.com/nginxinc/nginx-otel/archive/refs/tags/v%{version}.tar.gz#/nginx-otel-%{version}.tar.gz
Source101:      abseil-cpp-20211102.0.tar.gz
Source102:      grpc-1.46.7.tar.gz
Source103:      opentelemetry-cpp-1.18.0.tar.gz
Source104:      opentelemetry-proto-1.4.0.tar.gz
Source105:      protobuf-3.19.5.tar.gz

Patch100:       abseil-cpp_b957f0ccd00481cd4fd663d8320aa02ae0564f18.patch
Patch101:       abseil-cpp_4500c2fada4e952037c59bd65e8be1ba0b29f21e.patch
Patch102:       grpc_grpc-cmake-no-re2.patch


BuildRequires:  gcc
BuildRequires:  cmake
BuildRequires:  nginx-mod-devel
BuildRequires:  zlib-devel
BuildRequires:  pcre2-devel

%define WITH_CC_OPT $(echo %{optflags} $(pcre2-config --cflags))
%define WITH_LD_OPT -Wl,-z,relro -Wl,-z,now -Wl,-as-needed
%define bdir %{_builddir}/%{name}-%{version}

%description
Native OpenTelemetry (OTel) Module.
OpenTelemetry (OTel) is an observability framework for monitoring,
tracing, troubleshooting, and optimizing applications.
OTel enables the collection of telemetry data from a deployed application stack.

The ngx_otel_module dynamic module enables NGINX Open Source or
NGINX Plus to send telemetry data to an OTel collector.
It provides support for W3C trace context propagation,
OpenTelemetry Protocol (OTLP)/gRPC trace exports and offers
several benefits over exiting OTel modules, including:

Better Performance
3rd-party OTel implementations reduce performance of request
processing by as much as 50% when tracing is enabled.
The NGINX Native module limits this impact to approximately 10-15%.

Easy Provisioning
Setup and configuration can be done right in NGINX configuration files.

Dynamic, Variable-Based Control
The ability to control trace parameters dynamically using cookies,
tokens, and variables. Please see our Ratio-based Tracing example for more details.

%prep
%setup -qcTn %{name}-%{version}

tar xvzfo %{SOURCE100}
ln -s nginx-otel-* nginx-otel
tar xvzfo %{SOURCE101}
ln -s abseil-cpp-* abseil-cpp
tar xvzfo %{SOURCE102}
ln -s grpc-* grpc
tar xvzfo %{SOURCE103}
ln -s opentelemetry-cpp-* opentelemetry-cpp
tar xvzfo %{SOURCE104}
ln -s opentelemetry-proto-* opentelemetry-proto
tar xvzfo %{SOURCE105}
ln -s protobuf-* protobuf

cd %{bdir}/abseil-cpp
%patch100 -p1
cd %{bdir}/abseil-cpp
%patch101 -p1
cd %{bdir}/grpc
%patch102 -p1

%build
_nproc=`getconf _NPROCESSORS_ONLN`
if [ $_nproc -gt 1 ]; then
    _make_opts="-j$_nproc"
fi
cd %{bdir} && mkdir prebuilt && cd %{bdir}/abseil-cpp-20211102.0 && mkdir build && cd build && cmake -DCMAKE_CXX_STANDARD=17 -DCMAKE_CXX_EXTENSIONS=OFF -DCMAKE_CXX_VISIBILITY_PRESET=hidden -DCMAKE_POLICY_DEFAULT_CMP0063=NEW -DCMAKE_PREFIX_PATH=%{bdir}/prebuilt/ -DCMAKE_INSTALL_PREFIX:STRING=%{bdir}/prebuilt/ -DCMAKE_INSTALL_LIBDIR:STRING=lib -DCMAKE_POSITION_INDEPENDENT_CODE=ON -DBUILD_TESTING=OFF -DWITH_BENCHMARK=OFF -DCMAKE_BUILD_TYPE=RelWithDebInfo ../ && make $_make_opts install && cd %{bdir}/protobuf-3.19.5 && mkdir build && cd build && cmake -DCMAKE_CXX_STANDARD=17 -DCMAKE_CXX_EXTENSIONS=OFF -DCMAKE_CXX_VISIBILITY_PRESET=hidden -DCMAKE_POLICY_DEFAULT_CMP0063=NEW -DCMAKE_PREFIX_PATH=%{bdir}/prebuilt/ -DCMAKE_INSTALL_PREFIX:STRING=%{bdir}/prebuilt/ -DCMAKE_INSTALL_LIBDIR:STRING=lib -DCMAKE_POSITION_INDEPENDENT_CODE=ON -Dprotobuf_BUILD_TESTS=OFF -DCMAKE_BUILD_TYPE=RelWithDebInfo ../cmake/ && make $_make_opts install && cd %{bdir}/grpc-1.46.7 && mkdir build && cd build && CXXFLAGS='-DGRPC_NO_XDS -DGRPC_NO_RLS' cmake -DCMAKE_CXX_STANDARD=17 -DCMAKE_CXX_EXTENSIONS=OFF -DCMAKE_CXX_VISIBILITY_PRESET=hidden -DCMAKE_POLICY_DEFAULT_CMP0063=NEW -DCMAKE_PREFIX_PATH=%{bdir}/prebuilt/ -DCMAKE_INSTALL_PREFIX:STRING=%{bdir}/prebuilt/ -DCMAKE_INSTALL_LIBDIR:STRING=lib -DCMAKE_POSITION_INDEPENDENT_CODE=ON -DgRPC_BUILD_GRPC_RUBY_PLUGIN=OFF -DgRPC_BUILD_GRPC_PYTHON_PLUGIN=OFF -DgRPC_BUILD_GRPC_PHP_PLUGIN=OFF -DgRPC_BUILD_GRPC_OBJECTIVE_C_PLUGIN=OFF -DgRPC_BUILD_GRPC_NODE_PLUGIN=OFF -DgRPC_BUILD_GRPC_CSHARP_PLUGIN=OFF -DgRPC_BUILD_CSHARP_EXT=OFF -DgRPC_BUILD_CODEGEN=ON -DgRPC_SSL_PROVIDER=package -DgRPC_ZLIB_PROVIDER=package -DgRPC_CARES_PROVIDER=package -DgRPC_ABSL_PROVIDER=package -DgRPC_PROTOBUF_PROVIDER=package -DgRPC_PROTOBUF_PACKAGE_TYPE=CONFIG -DgRPC_USE_PROTO_LITE=ON -DCMAKE_BUILD_TYPE=RelWithDebInfo ../ && make $_make_opts install && cd %{bdir}/opentelemetry-cpp-1.18.0 && mkdir build && cd build && cmake -DCMAKE_CXX_STANDARD=17 -DCMAKE_CXX_EXTENSIONS=OFF -DCMAKE_CXX_VISIBILITY_PRESET=hidden -DCMAKE_POLICY_DEFAULT_CMP0063=NEW -DCMAKE_PREFIX_PATH=%{bdir}/prebuilt/ -DCMAKE_INSTALL_PREFIX:STRING=%{bdir}/prebuilt/ -DCMAKE_INSTALL_LIBDIR:STRING=lib -DCMAKE_POSITION_INDEPENDENT_CODE=ON -DBUILD_TESTING=OFF -DWITH_ABSEIL=ON -DWITH_BENCHMARK=OFF -DWITH_EXAMPLES=OFF -DCMAKE_BUILD_TYPE=RelWithDebInfo ../ && make $_make_opts install || exit 1
cd %{bdir}

export PATH=%{bdir}/prebuilt/bin/:$PATH
export NGX_OTEL_PROTO_DIR=%{bdir}/opentelemetry-proto-1.4.0
export CMAKE_PREFIX_PATH=%{bdir}/prebuilt/
export NGX_OTEL_CMAKE_OPTS="-DNGX_OTEL_GRPC=package -DNGX_OTEL_SDK=package -DNGX_OTEL_PROTO_DIR=$NGX_OTEL_PROTO_DIR"

%undefine _strict_symbol_defs_build
cp -a "%{_nginx_srcdir}" "%{_nginx_buildsrcdir}"
cd "%{_nginx_buildsrcdir}"
./configure --with-compat --add-dynamic-module=$(realpath %{_nginx_modsrcdir}/nginx-otel-%{version}) \
            --with-cc-opt="%{WITH_CC_OPT} " \
            --with-ld-opt="%{WITH_LD_OPT} "
make %{?_smp_mflags} modules


%install
cd %{bdir}
%{__rm} -rf %{buildroot}
%{__mkdir} -p %{buildroot}%{nginx_moddir}
for so in `find %{bdir}/ -type f -name "*.so"`; do
%{__install} -m755 $so %{buildroot}%{nginx_moddir}
done

%{__install} -dm 0755 %{buildroot}%{nginx_modconfdir}
echo 'load_module "%{nginx_moddir}/ngx_otel_module.so";'    > %{buildroot}%{nginx_modconfdir}/mod-otel.conf

%files
%license nginx-otel/LICENSE
%doc nginx-otel/{CONTRIBUTING.md,README.md,CODE_OF_CONDUCT.md,SECURITY.md}
%{nginx_moddir}/*.so
%{nginx_modconfdir}/mod-otel.conf

%changelog
* Mon Mar 17 2025 Teddysun <i@teddysun.com> - 0.1.2-1
- Initial packaging version 0.1.2 for nginx 1.27.4
